<?php

namespace Drupal\castorcito\Form;

use Drupal\castorcito\CastorcitoComponentFieldManager;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Controller for castorcito component edit form.
 *
 * @internal
 */
class CastorcitoComponentEditForm extends CastorcitoComponentFormBase {

  /**
   * The castorcito component field manager service.
   *
   * @var \Drupal\castorcito\CastorcitoComponentFieldManager
   */
  protected $componentFieldManager;

  /**
   * Constructs an CastorcitoComponentEditForm object.
   *
   * @param \Drupal\Core\Entity\EntityStorageInterface $component_storage
   *   The entity storage.
   * @param \Drupal\castorcito\CastorcitoComponentFieldManager $component_field_manager
   *   The castorcito component field manager service.
   */
  public function __construct(EntityStorageInterface $component_storage, CastorcitoComponentFieldManager $component_field_manager) {
    parent::__construct($component_storage);
    $this->componentFieldManager = $component_field_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')->getStorage('castorcito_component'),
      $container->get('plugin.manager.castorcito_component_field')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);
    $cfm = $this->componentFieldManager->getDefinitions();
    $model = $this->entity->getModel();
    $new_field_options = [];

    if (!empty($cfm)) {
      foreach ($cfm as $field => $definition) {
        $new_field_options[$field] = $definition['label'];
      }
    }

    $form['#title'] = $this->t('Edit %label component', [
      '%label' => $this->entity->label(),
    ]);

    $form['#attached']['library'][] = 'castorcito/castorcito.admin';

    if (!$this->entity->get('inside_container')) {
      $form['components_display_settings'] = [
        '#type' => 'details',
        '#title' => $this->t('Display settings'),
      ];

      $form['components_display_settings']['html'] = [
        '#markup' => $this->t('<p>With the <strong>Add option</strong> button, you can add configuration options for the component display, such as the slideshow speed or the component\'s width. These settings can be overridden in the <em>Manage Display</em> section of the field in Drupal.</p>'),
      ];

      $form['components_display_settings']['add_display_settings_option'] = [
        '#type' => 'link',
        '#title' => $this->t('Add option'),
        '#url' => Url::fromRoute('castorcito.component_display_settings', [
          'castorcito_component' => $this->entity->id(),
        ]),
        '#attributes' => ['class' => ['button']],
      ];

      if ($display_settings = $this->entity->get('display_settings')) {
        $form['components_display_settings']['#open'] = TRUE;
        $form['components_display_settings']['components_display_settings_table'] = [
          '#type' => 'table',
          '#header' => [
            $this->t('Options'),
            $this->t('Actions'),
          ],
          '#empty' => $this->t('Currently there are no display settings options for the component.'),
        ];

        foreach ($display_settings as $ds_key => $setting) {
          $element = [
            '#type' => $setting['type'],
            '#title' => $this->t($setting['label']),
            '#default_value' => $setting['default_value'] ?? '',
            '#description' => $setting['description'] ?? '',
            '#required' => $setting['required'] ?? '',
          ];

          if ($setting['type'] === 'select' && !empty($setting['options'])) {
            $element['#options'] = array_column($setting['options'], 'label', 'key');
          }

          $form['components_display_settings']['components_display_settings_table'][$ds_key]['default_value'] = $element;
          $form['components_display_settings']['components_display_settings_table'][$ds_key]['buttons'] = [
            '#type' => 'container',
          ];

          $form['components_display_settings']['components_display_settings_table'][$ds_key]['buttons']['edit'] = [
            '#type' => 'link',
            '#title' => $this->t('Edit'),
            '#url' => Url::fromRoute('castorcito.component_display_settings', [
              'castorcito_component' => $this->entity->id(),
              'field_name' => $ds_key,
            ]),
            '#attributes' => ['class' => ['button']],
          ];

          $form['components_display_settings']['components_display_settings_table'][$ds_key]['buttons']['delete'] = [
            '#type' => 'link',
            '#title' => $this->t('Delete'),
            '#url' => Url::fromRoute('castorcito.component_display_settings_delete', [
              'castorcito_component' => $this->entity->id(),
              'field_name' => $ds_key,
            ]),
            '#attributes' => ['class' => ['button']],
          ];
        }
      }
    }

    $form['components_field'] = [
      '#type' => 'details',
      '#title' => $this->t('Fields'),
      '#open' => TRUE,
    ];

    $form['components_field']['component_item'] = [
      '#type' => 'item',
      '#title' => 'Add field',
      '#description' => $this->t('Shows all the fields defined in the Castorcito module plugin, if you want to add more fields you can do it by creating your own custom module.'),
    ];

    $form['components_field']['component_item']['new_field'] = [
      '#type' => 'select',
      '#title' => '',
      '#options' => $new_field_options,
      '#empty_option' => $this->t('- Select a field type -'),
      '#prefix' => '<div class="cc-field-new clearfix">',
    ];

    $form['components_field']['component_item']['new_add'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add'),
      '#validate' => ['::componentFieldValidate'],
      '#submit' => ['::componentFieldConfiguration'],
      '#suffix' => '</div>',
    ];

    $form['components_field']['component_field_table'] = [
      '#type' => 'table',
      '#header' => [
        $this->t('Label'),
        $this->t('Machine name'),
        $this->t('Field type'),
        $this->t('Weight'),
        $this->t('Operations'),
      ],
      '#tabledrag' => [
        [
          'action' => 'order',
          'relationship' => 'sibling',
          'group' => 'component-field-order-weight',
        ],
      ],
      '#empty' => $this->t('There are currently no fields for the component. Add by selecting a field in the "Add field" option.'),
      '#weight' => 5,
    ];

    if (!empty($model['fields'])) {
      $number = 0;

      foreach ($model['fields'] as $key => $value) {
        $field = $this->entity->getComponentField($key);
        $form['components_field']['component_field_table'][$key]['#attributes']['class'][] = 'draggable';
        $form['components_field']['component_field_table'][$key]['label'] = [
          '#tree' => FALSE,
          'data' => [
            'label' => [
              '#plain_text' => $field->getFieldLabel(),
            ],
          ],
        ];

        $form['components_field']['component_field_table'][$key]['machine_name'] = [
          '#tree' => FALSE,
          'data' => [
            'label' => [
              '#plain_text' => $field->getFieldName(),
            ],
          ],
        ];

        $form['components_field']['component_field_table'][$key]['field_type'] = [
          '#tree' => FALSE,
          'data' => [
            'label' => [
              '#plain_text' => $field->getPluginId(),
            ],
          ],
        ];

        $form['components_field']['component_field_table'][$key]['weight'] = [
          '#type' => 'weight',
          '#title' => $this->t('Weight for'),
          '#title_display' => 'invisible',
          '#default_value' => $number,
          '#attributes' => [
            'class' => ['component-field-order-weight'],
          ],
        ];

        $operations['edit'] = [
          'title' => $this->t('Edit'),
          'url' => Url::fromRoute('castorcito.component_field_edit_form', [
            'castorcito_component' => $this->entity->id(),
            'field_type' => $key,
          ]),
        ];

        $operations['delete'] = [
          'title' => $this->t('Delete'),
          'url' => Url::fromRoute('castorcito.component_field_delete_form', [
            'castorcito_component' => $this->entity->id(),
            'field_type' => $key,
          ]),
        ];

        $form['components_field']['component_field_table'][$key]['operations'] = [
          '#type' => 'operations',
          '#links' => $operations,
        ];

        $number++;
      }
    }

    return $form;
  }

  /**
   * Validate handler for component field.
   */
  public function componentFieldValidate($form, FormStateInterface $form_state) {
    if (!$form_state->getValue('new_field')) {
      $form_state->setErrorByName('new_field', $this->t('Select the field to add.'));
    }
  }

  /**
   * Submit handler for component field.
   */
  public function componentFieldConfiguration($form, FormStateInterface $form_state) {

    $form_state->setRedirect(
      'castorcito.component_field_add_form',
      [
        'castorcito_component' => $this->entity->id(),
        'field_type' => $form_state->getValue('new_field'),
      ],
    );
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $this->entity->setBaseModel($form_state->getValue('label'));

    if (!empty($form_state->getValue('component_field_table'))) {
      $this->orderModel($form_state->getValue('component_field_table'));
    }

    if (!empty($form_state->getValue('components_display_settings_table'))) {
      $this->entity->setComponentDisplaySettingsValues($form_state->getValue('components_display_settings_table'));
    }

    parent::save($form, $form_state);
    $this->messenger()->addStatus($this->t('Changes to the component have been saved.'));
  }

  /**
   * {@inheritdoc}
   */
  private function orderModel($order) {
    $model = $this->entity->getModel();
    $model['fields'] = array_merge($order, $model['fields']);
    $this->entity->setModel($model);
  }

}
