<?php

namespace Drupal\castorcito\Plugin\Field\FieldFormatter;

use Drupal\castorcito\CastorcitoManager;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Component\Serialization\Json;

/**
 * Plugin implementation of the 'castorcito_component_formatter' formatter.
 *
 * @FieldFormatter(
 *   id = "castorcito_component_formatter",
 *   label = @Translation("Castorcito component"),
 *   field_types = {
 *     "json",
 *     "json_native",
 *     "json_native_binary",
 *   }
 * )
 */
class CastorcitoComponentFormatter extends FormatterBase {

  /**
   * The config factory.
   *
   * @var Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * The castorcito manager.
   *
   * @var Drupal\castorcito\CastorcitoManager
   */
  protected $castorcitoManager;

  /**
   * The params widget config name.
   *
   * @var array
   */
  protected $paramsWidgetConfigName = [];

  /**
   * Constructs a new CastorcitoComponentFormatter.
   *
   * @param string $plugin_id
   *   The plugin_id for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the formatter is associated.
   * @param array $settings
   *   The formatter settings.
   * @param string $label
   *   The formatter label display setting.
   * @param string $view_mode
   *   The view mode.
   * @param array $third_party_settings
   *   Third party settings.
   * @param Drupal\Core\Config\ConfigFactory $config_factory
   *   The config factory.
   * @param Drupal\castorcito\CastorcitoManager $castorcito_manager
   *   The castorcito manager.
   */
  public function __construct(
    $plugin_id,
    $plugin_definition,
    FieldDefinitionInterface $field_definition,
    array $settings,
    $label,
    $view_mode,
    array $third_party_settings,
    ConfigFactory $config_factory,
    CastorcitoManager $castorcito_manager,
  ) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
    $this->configFactory = $config_factory;
    $this->castorcitoManager = $castorcito_manager;
    $this->paramsWidgetConfigName = [
      'entity_type' => $this->fieldDefinition->get('entity_type'),
      'bundle' => $this->fieldDefinition->get('bundle'),
      'field_name' => $this->fieldDefinition->getName(),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('config.factory'),
      $container->get('castorcito.manager')
    );
  }

  /**
   * {@inheritdoc}
  */
  public static function defaultSettings() {
    return [
      'display_settings_components' => [],
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
  */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $display_settings_components = $this->getSetting('display_settings_components');
    $config_widget_name = $this->castorcitoManager->configWidgetName($this->paramsWidgetConfigName);
    $config_widget_field = $this->configFactory->getEditable($config_widget_name);
    $components = $config_widget_field->get('components');
    $allowed_components = $this->castorcitoManager->castorcitoComponents($components);

    $form['display_settings_components'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Display settings'),
    ];

    foreach ($allowed_components as $c_key => $component) {
      if ($settings = $component->get('display_settings')) {
        $form['display_settings_components'][$c_key] = [
          '#type' => 'details',
          '#title' => $component->label(),
          '#states' => [
            'visible' => [
              ':input[value="' . $c_key . '"]' => ['checked' => TRUE],
            ],
          ],
        ];

        foreach ($settings as $s_key => $setting) {
          $element = [
            '#type' => $setting['type'],
            '#title' => $this->t($setting['label']),
            '#default_value' => $display_settings_components[$c_key][$s_key] ?? $setting['default_value'],
            '#description' => $setting['description'] ?? '',
            '#required' => $setting['required'] ?? '',
          ];

          if ($setting['type'] === 'select' && !empty($setting['options'])) {
            $element['#options'] = array_column($setting['options'], 'label', 'key');
          }

          $form['display_settings_components'][$c_key][$s_key] = $element;
        }
      }

      $this->fieldDisplayElements($form, $c_key, $component->get('field_configuration'));
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $display_settings_components = $this->getSetting('display_settings_components');
    $summary = [];

    if (!empty($display_settings_components)) {
      $components_options = $this->castorcitoManager->castorcitoComponentsOptions();
      $items = [];
      foreach ($display_settings_components as $con_key => $config) {
        $items[] = $components_options[$con_key];
      }

      $summary[] = $this->t('<strong>Display settings</strong>');
      $summary[] = [
        '#theme' => 'item_list',
        '#list_type' => 'ul',
        //'#title' => $this->t('Display settings'),
        '#items' => $items,
      ];
    }
    else {
      $summary[] = $this->t('Default settings.');
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $display_settings_components = $this->getSetting('display_settings_components');
    $elements = [];

    if (!empty($items)) {
      foreach ($items as $key => $item) {
        $model = Json::decode($item->value);
        $component = $this->castorcitoManager->castorcitoComponent($model['type']);
        $display_settings = [];

        if ($settings = $component->get('display_settings')) {
          foreach ($settings as $s_key => $setting) {
            $display_settings[$s_key] = $display_settings_components[$model['type']][$s_key] ?? $setting['default_value'];
          }
        }

        if ($field_configurations = $component->get('field_configuration')) {
          foreach ($field_configurations as $f_key => $configuration) {
            if (isset($configuration['settings']['display'])) {
              $display_settings['fields'][$f_key]['display'] = $display_settings_components[$model['type']][$f_key]['display'] ?? $configuration['settings']['display'];
            }
          }
        }

        $elements[] = [
          '#theme' => 'castorcito_component_formatter',
          '#component_data' => $model,
          '#display_settings' => $display_settings,
        ];
      }
    }

    return $elements;
  }

  /**
   * Function to build element field display.
   */
  public function fieldDisplayElements(array &$form, $component_id, array $field_configurations) {
    if (empty($field_configurations)) {
      return $form;
    }

    $image_style_options = image_style_options(FALSE);
    $entity_display_repository = \Drupal::service('entity_display.repository');
    $formatter_settings = $this->getSetting('display_settings_components');
    foreach ($field_configurations as $f_key => $configuration) {
      if (!isset($configuration['settings']['display'])) {
        continue;
      }

      if (isset($configuration['settings']['display']['view_mode'])) {
        $view_mode_options = $entity_display_repository->getViewModeOptions($configuration['settings']['entity_type']);
        $form['display_settings_components'][$component_id][$f_key]['display']['view_mode'] = [
          '#type' => 'select',
          '#title' => $this->t($configuration['label'] . ': View mode'),
          '#options' => $view_mode_options,
          '#default_value' => $formatter_settings[$component_id][$f_key]['display']['view_mode'] ?? $configuration['settings']['display']['view_mode'],
          '#empty_option' => $this->t('- Select -'),
        ];
      }
      
      if (isset($configuration['settings']['display']['image_style'])) {
        $form['display_settings_components'][$component_id][$f_key]['display']['image_style'] = [
          '#type' => 'select',
          '#title' => $this->t($configuration['label'] . ': Image style'),
          '#options' => $image_style_options,
          '#default_value' => $formatter_settings[$component_id][$f_key]['display']['image_style'] ?? $configuration['settings']['display']['image_style'],
          '#empty_option' => $this->t('None (original image)'),
        ];
      }
    }
  }

}
